DRIVER 'org.apache.derby.jdbc.ClientDriver';
CONNECT 'jdbc:derby://localhost:1527/BookStore;create=true' USER 'guest' PASSWORD 'password';

DROP TABLE Inventory;
DROP TABLE BookAuthors;
DROP TABLE ElectronicBook;
DROP TABLE Book;
DROP TABLE Store;

CREATE TABLE Store (
    STORE_ID		BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
    NickName	VARCHAR(40),
    Street1     VARCHAR(40),
    Street2		VARCHAR(40),
    City		VARCHAR(40),
    State		VARCHAR(2),
    ZipCode     VARCHAR(10),
    Version		INTEGER,
    CONSTRAINT	PK_STORE PRIMARY KEY(STORE_ID)
);

CREATE TABLE Book (
   BOOK_ID		BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
   ISBN			VARCHAR(10) NOT NULL,
   Title		VARCHAR(40),
   PublishedOn	DATE,
   Price		DECIMAL(5,2),
   Version		INTEGER,
   CONSTRAINT	PK_BOOK PRIMARY KEY(BOOK_ID)
);

CREATE TABLE BookAuthors (
   BOOK_ID		BIGINT NOT NULL,
   FirstName	VARCHAR(40) NOT NULL,
   LastName		VARCHAR(40) NOT NULL,
   CONSTRAINT	PK_BOOK_AUTHORS  PRIMARY KEY(BOOK_ID, FirstName, LastName),
   CONSTRAINT	FK_BOOK_AUTHORS_FOR_BOOK FOREIGN KEY(BOOK_ID) REFERENCES Book(BOOK_ID)
);

CREATE TABLE ElectronicBook (
   BOOK_ID		BIGINT NOT NULL,
   URL			VARCHAR(120) NOT NULL,
   FileType		VARCHAR(40) NOT NULL,
   CONSTRAINT	PK_ELECTRONIC_BOOK PRIMARY KEY(BOOK_ID),
   CONSTRAINT	FK_ELECTRONIC_BOOK_AND_BOOK FOREIGN KEY(BOOK_ID) REFERENCES Book(BOOK_ID)
);

CREATE TABLE Inventory (
   INVENTORY_ID	BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1, INCREMENT BY 1),
   BOOK_ID		BIGINT NOT NULL,
   STORE_ID		BIGINT NOT NULL,
   Quantity		INTEGER NOT NULL,
   Version		INTEGER,
   CONSTRAINT	PK_INVENTORY        PRIMARY KEY(INVENTORY_ID),
   CONSTRAINT	UNIQUE_BOOK_STORE   UNIQUE(BOOK_ID, STORE_ID),
   CONSTRAINT	FK_INVENTORY_FOR_BOOK FOREIGN KEY(BOOK_ID) REFERENCES Book(BOOK_ID),
   CONSTRAINT	FK_INVENTORY_STORE FOREIGN KEY(STORE_ID) REFERENCES Store(STORE_ID)
);